#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##################################
# AUTHOR:   p-tank <akazian@student.42.fr>
# FILE:     main.py
# CREATED:  19:02:34 09/06/2014
# MODIFIED: 19:02:34 09/06/2014
##################################

import argparse
import sys
from stalker42.stalker import Stalker42
import getpass
import time
import os
from stalker42.__init__ import __version__

if sys.platform == 'linux':
    import dbus

def option():
    parser = argparse.ArgumentParser(description="Stalk 42's students")
    information = parser.add_argument_group('information')
    contact_group = parser.add_mutually_exclusive_group()
    parser.add_argument('-v', '--version', action='version', version='%(prog)s: ' + __version__)
    parser.add_argument('contact', nargs='*', default=[],
                         help="list name to stalk")
    information.add_argument('-t', "--tel", action='store_true', default=False, dest="tel",
                             help="mobile-phone, requires connection")
    information.add_argument('-m', '--mail', action='store_true', default=False, dest="mail",
                            help="e-mail, requires connection if not at school")
    information.add_argument('-p', '--place', action='store_true', default=True, dest="place")
    contact_group.add_argument('-a', '--add', action='store_true', default=False, dest="add",
                            help='add contact to contact [contact] "name" -s "status"')
    contact_group.add_argument('-d', '--del', action='store_true', default=False,  dest="delete",
                            help='del contact to contact list [contact] "name"')
    parser.add_argument('-q', '--quiet', action='store_true', default=False,  dest="quiet")
    parser.add_argument('-s', '--status', action='store',  dest="status",
                            help='status')
    parser.add_argument('-u', '--user', action='store',  dest="user",
                            help='user')
    parser.add_argument('-n', '--notify', nargs='?',const=20, type=int, dest="time",
                            help='pop notify by search every time')
    information.add_argument('-c', '--corrector', action='store_true', default=False, dest="corrector",
                            help="find corrector, requires connection")

    return (parser.parse_args())

if sys.platform == 'linux':
    def notify(summary='', body='', app_name='', app_icon='', timeout=5000, actions=[], hints=[], replaces_id=0):
    	_bus_name = 'org.freedesktop.Notifications'
    	_object_path = '/org/freedesktop/Notifications'
    	_interface_name = _bus_name

    	session_bus = dbus.SessionBus()
    	obj = session_bus.get_object(_bus_name, _object_path)
    	interface = dbus.Interface(obj, _interface_name)
    	interface.Notify(app_name, replaces_id, app_icon, summary, body, actions, hints, timeout)
elif sys.platform == 'darwin':
    def notify(summary='', subtitle='', body=''):
        t = '-title {!r}'.format(summary)
        s = '-subtitle {!r}'.format(subtitle)
        m = '-message {!r}'.format(body)
        os.system('terminal-notifier {}'.format(' '.join([m, t, s])))

def notify_place(last, contact):
    for c in last:
        for n in contact:
           if c['name'] == n['name']:
                if c['place'] != n['place']:
                     if c['place'] == "Not found":
                         notify(summary=''.join(c['name']), body=' '.join(" log at " + n['place'] + " " + c['status']))
                     else:
                         notify(summary="".join(c['name']), body=' '.join("  log out" + " " + c['status']))
                break


def print_contact_info(opt, contact):
    for name in contact:
        print("\033[1;33m" + name['name'] + " as " + name['status'] + "\033[0m")
        if opt.place:
            print("place: " + name['place'])
        if opt.mail:
            print("mail: " + name['mail'])
        if opt.tel:
            print("tel: " + name['tel'])
        if opt.corrector:
            print('project: ' + name['project'])

def connect_to_stalk(stalker, user, opt, school):
     pwd = getpass.getpass('password: ')
     try:
        if not school:
            stalker.connectDashboard(user, pwd)
        stalker.connectLdap(user, pwd)
        if opt.corrector:
            stalker.connectIntra(user, pwd)
     except:
        print(os.path.basename(sys.argv[0]) + ": Connection fail")

def main():
    opt = option()
    contact_info = None
    user = None
    contactfile = os.getenv("HOME") + "/.contact.csv"
    school = False
    try:
        if os.getenv("MAIL").split('@')[1] == 'student.42.fr':
            school = True
    except:
        school = False
    stalker = Stalker42(contactfile)
    if opt.user:
        user = opt.user
        connect_to_stalk(stalker, user, opt, school)
    elif opt.mail and school:
        stalker.connectLdapSchool()
    if opt.add:
        if opt.status == None:
            status = "student"
        else:
            status = opt.status
        stalker.addContact(opt.contact, status)
        exit(0)
    elif opt.delete:
        stalker.delContact(opt.contact)
        exit(0)
# here loop to -n --notify <time>
    last = {}
    while 42:
        try:
            if opt.corrector:
                if user == None:
                    user = input('Login: ')
                    connect_to_stalk(stalker, user, opt, school)
                contact_info = stalker.findCorrector()
            elif len(opt.contact) == 0 and opt.status != None:
                contact_info = stalker.findContact(opt.status)
            elif len(opt.contact) != 0:
                contact_info = stalker.findContact(status="student", contact=opt.contact)
            elif len(opt.contact) == 0 and opt.status == None:
                contact_info = stalker.findContact()
            if contact_info:
                if opt.quiet == False:
                    print_contact_info(opt, contact_info)
            else:
                print(os.path.basename(sys.argv[0]) + ": No contact found")
            if not opt.time:
                exit(0)
            if not last:
                last = contact_info
            else:
                notify_place(last, contact_info)
                last = contact_info
            time.sleep(opt.time)
        except KeyboardInterrupt:
            print("\nBye")
            exit(0)

if __name__ == '__main__':
    main()
